﻿-- Author      : Deldinor of Elune
-- Create Date : 2/1/2011 7:31:49 AM

addonName = "DailyGrind";
addonFullName = "Daily Grind";

DailyGrind = LibStub("AceAddon-3.0"):NewAddon(addonName, "AceConsole-3.0", "AceEvent-3.0");
DailyGrind.version = GetAddOnMetadata(addonName, "Version");

BINDING_HEADER_DAILYGRIND = addonFullName;

local titleColor = "FF70B7FF";
addonTitle = "|c"..titleColor..addonFullName.."|r";
addonAbbr = "|c"..titleColor.."[DG]|r";
commandColor = "FFFFC654";
local disabledText = "|cFFFF0000DISABLED|r";
local enabledText = "|cFF00FF00ENABLED|r";
maxQuests = 25;

local currentDaily;

function DailyGrind:OnInitialize()
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(addonName, self.Options)
	local configDialog = LibStub("AceConfigDialog-3.0")
	self.optionsFrames = {}
	self.optionsFrames.DailyGrind = configDialog:AddToBlizOptions(addonName, addonName, nil, "General")
	
    self:RegisterChatCommand("dailygrind", "ChatCommand");
    self:RegisterChatCommand("dg", "ChatCommand");
    self:InitializeSettings();
end

function DailyGrind:InitializeSettings()
	if DailyGrindSettings == nil then
		DailyGrindSettings = self.defaultSettings;
	end
	DailyGrindSettings.Version = nil;
	if DailyGrindQuests == nil then
		DailyGrindQuests = {};
	end
	if DailyGrindSettings.NpcBlacklist == nil then
		DailyGrindSettings.NpcBlacklist = {};
	end
end

DailyGrind.defaultSettings = {
	Enabled = true,
	Blacklist = {},
	NpcBlacklist = {},
	Rewards = {},
	AutoAcceptAllEnabled = false
}

function DailyGrind:OnEnable()
    self:RegisterEvent("GOSSIP_SHOW");
    self:RegisterEvent("QUEST_GREETING");
    self:RegisterEvent("QUEST_DETAIL");
    self:RegisterEvent("QUEST_PROGRESS");
    self:RegisterEvent("QUEST_COMPLETE");
end

function DailyGrind:ChatCommand(input)
	local args = { strsplit(" ", input:trim(), 3) };
	local command = args[1]:lower();
	 
	if command == "" then
		self:ShowOptions();
	
	elseif command == "import" then
		self:Import();
		
	elseif command == "reset" then
		-- TODO: Implement confirmation.
		self:Reset();
		
  	elseif command == "enable" or command == "on" then
		self:SetEnabled(true);
		print(addonAbbr.." "..self:GetStatus());
		
	elseif command == "disable" or command == "off" then
		self:SetEnabled(false);
		print(addonAbbr.." "..self:GetStatus());
		
	elseif command == "toggle" or command == "tog" then
		self:Toggle();
		print(addonAbbr.." "..self:GetStatus());
		
	elseif command == "all" then
		self:ToggleAutoAcceptAll();
		print(addonAbbr.." "..self:GetAutoAcceptAllStatus());
		
	elseif command == "blacklist" or command == "bl" then
		self:HandleBlacklist(args);
	
	elseif command == "npcblacklist" or command == "nbl" then
		self:HandleNpcBlacklist(args);
		
	elseif command == "rewardlist" or command == "rl" then
		self:HandleRewardList(args);
	
	elseif command == "trim" then
		self:TrimList();

	else
		self:ShowHelp();
		
    end
end

function DailyGrind:Import()
	print(addonAbbr.." Import feature has been deprecated. Use the new Auto-Accept All feature (/dg all).");
end

function DailyGrind:Reset()
	DailyGrindQuests = {};
	print(addonAbbr.." Quest history reset.");
end

function DailyGrind:Toggle()
	self:SetEnabled(not self:GetEnabled());
end

function DailyGrind:SetEnabled(state)
	DailyGrindSettings.Enabled = state;
end

function DailyGrind:GetEnabled()
	return DailyGrindSettings.Enabled;
end

function DailyGrind:ToggleAutoAcceptAll()
	self:SetAutoAcceptAllEnabled(not self:GetAutoAcceptAllEnabled());
end

function DailyGrind:SetAutoAcceptAllEnabled(state)
	DailyGrindSettings.AutoAcceptAllEnabled = state;
end

function DailyGrind:GetAutoAcceptAllEnabled()
	return DailyGrindSettings.AutoAcceptAllEnabled;
end

function DailyGrind:GetStatus()
	if self:GetEnabled() then
		return addonTitle.." - "..enabledText;
	else
		return addonTitle.." - "..disabledText;
	end
end

function DailyGrind:GetAutoAcceptAllStatus()
	local all = "Auto-accept all quests option is ";
	if self:GetAutoAcceptAllEnabled() then
		return all..enabledText;
	else
		return all..disabledText;
	end
end

function DailyGrind:GetNumPlayerActiveQuests()
	local numEntries, numQuests = GetNumQuestLogEntries();
	return numQuests;
end

function DailyGrind:GetAvailableQuestTitleIndex(index)
	return self:GetAvailableQuestOffset() * (index - 1) + 1;
end

function DailyGrind:GetActiveQuestTitleIndex(index)
	return self:GetActiveQuestOffset() * (index - 1) + 1;
end

function DailyGrind:GetAvailableQuestOffset()
	return 6;
end

function DailyGrind:GetActiveQuestOffset()
	return 5;
end

function DailyGrind:IsDailyActiveQuest(activeQuestTitle)
	for i = 1, self:GetNumPlayerActiveQuests(), 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if self:SanitizeQuestTitle(questTitle) == activeQuestTitle then
			return isDaily == 1;
		end
	end
end

function DailyGrind:IsDailyAvailableQuest_Gossip(availableQuests, index)
	local titleIdx = self:GetAvailableQuestTitleIndex(index);
	local isDailyIdx = titleIdx + 3;
	local isRepeatableIdx = titleIdx + 4;
	return availableQuests[isDailyIdx] == 1 or availableQuests[isRepeatableIdx] == 1
end

function DailyGrind:IsDailyAvailableQuest_Greeting(availableQuestInfo)
	local isTrivial, isDaily, isRepeatable = {availableQuestInfo};
	return isDaily == 1 or isRepeatable == 1;
end

function DailyGrind:GetSanitizedAvailableQuestTitle(availableQuests, index)
	local titleIdx = self:GetAvailableQuestTitleIndex(index);
	return self:SanitizeQuestTitle(availableQuests[titleIdx]);
end

function DailyGrind:GetSanitizedActiveQuestTitle(activeQuests, index)
	local titleIdx = self:GetActiveQuestTitleIndex(index);
	return self:SanitizeQuestTitle(activeQuests[titleIdx]);
end

function DailyGrind:IsCompleteActiveQuest_Gossip(activeQuests, index)
	local titleIdx = self:GetActiveQuestTitleIndex(index)
	local isCompleteIdx = titleIdx + 3;
	return activeQuests[isCompleteIdx] == 1;
end

function DailyGrind:IsCompleteActiveQuest_Greeting(activeQuestTitle)
	for i = 1, self:GetNumPlayerActiveQuests(), 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if self:SanitizeQuestTitle(questTitle) == activeQuestTitle then
			return isComplete == 1;
		end
	end
end

function DailyGrind:GOSSIP_SHOW()
	if not DailyGrindSettings.Enabled or IsControlKeyDown() then
		return;
	end
	
	local numAvailableQuests = GetNumGossipAvailableQuests();
	local numActiveQuests = GetNumGossipActiveQuests();
	
	local npcName, realm = UnitName("npc");
	if (self:IsBlacklistedNpc(npcName)) then
		if (numAvailableQuests > 0) then
			print(addonAbbr.." "..npcBlacklistText..": Ignoring quests from \""..npcName.."\""); 
		end
		return;
	end
	
	-- Turn in known completed quests.
	if numActiveQuests > 0 then
		local activeQuests = {GetGossipActiveQuests()};
		for i = 1, numActiveQuests, 1 do
		
			local questTitle = self:GetSanitizedActiveQuestTitle(activeQuests, i);
			local isComplete = self:IsCompleteActiveQuest_Gossip(activeQuests, i)
			local isDaily = self:IsDailyActiveQuest(questTitle);
			local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
			
			if isComplete and (isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily)) then 
				if self:IsBlacklisted(questTitle) then
					print(addonAbbr.." "..blacklistText..": Skipped \""..questTitle.."\"");
				else
					SelectGossipActiveQuest(i);
				end
			end
		end
	end
	
	-- Accept known available quests.
	if numAvailableQuests > 0 then
		local availableQuests = {GetGossipAvailableQuests()};
		for i = 1, numAvailableQuests, 1 do
		
			local questTitle = self:SanitizeQuestTitle(availableQuests[self:GetAvailableQuestTitleIndex(i)]);
			local isDaily = self:IsDailyAvailableQuest_Gossip(availableQuests, i);
			local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
			
			if isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily) then
				if self:IsBlacklisted(questTitle) then
					print(addonAbbr.." "..blacklistText..": Skipped \""..questTitle.."\"");
				else
					if self:GetNumPlayerActiveQuests() < maxQuests then
						SelectGossipAvailableQuest(i);
					else
						print(addonAbbr.." Cannot auto-accept \""..questTitle.."\": Quest log is full.");
					end
				end
			end
		end
	end
	
end

function DailyGrind:QUEST_GREETING() 
	if not DailyGrindSettings.Enabled or IsControlKeyDown() then 
		return; 
	end 
	
	local numAvailableQuests = GetNumAvailableQuests(); 
	local numActiveQuests = GetNumActiveQuests(); 
	
	local npcName, realm = UnitName("npc");
	if (self:IsBlacklistedNpc(npcName)) then
		if (numAvailableQuests > 0) then
			print(addonAbbr.." "..npcBlacklistText..": Ignoring quests from \""..npcName.."\""); 
		end
		return;
	end
	
	-- Turn in known completed quests. 
	if numActiveQuests > 0 then 
		for i = 1, numActiveQuests, 1 do
		
			local questTitle = self:SanitizeQuestTitle(GetActiveTitle(i));
			local isComplete = self:IsCompleteActiveQuest_Greeting(questTitle)
			local isDaily = self:IsDailyActiveQuest(questTitle);
			local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
			
			if isComplete and (isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily)) then 
				if self:IsBlacklisted(questTitle) then 
					print(addonAbbr.." "..blacklistText..": Skipped \""..questTitle.."\""); 
				else 
					SelectActiveQuest(i); 
				end 
			end 
		end 
	end 

	-- Accept known available quests.
	if numAvailableQuests > 0 then 
		for i = 1, numAvailableQuests, 1 do 
		
			local questTitle = self:SanitizeQuestTitle(GetAvailableTitle(i));
			local isDaily = self:IsDailyAvailableQuest_Greeting(GetAvailableQuestInfo(i));
			local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
			
			if isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily) then
				if self:IsBlacklisted(questTitle) then 
					print(addonAbbr.." "..blacklistText..": Skipped \""..questTitle.."\""); 
				else 
					if self:GetNumPlayerActiveQuests() < maxQuests then 
						SelectAvailableQuest(i); 
					else 
						print(addonAbbr.." Cannot auto-accept \""..questTitle.."\": Quest log is full."); 
					end 
				end
			end
		end
	end
	
end

function DailyGrind:QUEST_DETAIL()
	local questTitle = self:SanitizeQuestTitle(GetTitleText());
	local npcName, realm = UnitName("npc");
	
	if not DailyGrindSettings.Enabled or IsControlKeyDown() or self:IsBlacklisted(questTitle) or self:IsBlacklistedNpc(npcName) then
		return;
	end
	
	local isDaily = QuestIsDaily() == 1 or QuestIsWeekly() == 1;
	local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
		
	if isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily) then
		if self:GetNumPlayerActiveQuests() < maxQuests then
			AcceptQuest();
		else
			print(addonAbbr.." Cannot auto-accept \""..questTitle.."\": Quest log is full.");
		end
	end
end

function DailyGrind:QUEST_PROGRESS()
	local questTitle = self:SanitizeQuestTitle(GetTitleText());
	local npcName, realm = UnitName("npc");
	
	if not DailyGrindSettings.Enabled or IsControlKeyDown() or self:IsBlacklisted(questTitle) or self:IsBlacklistedNpc(npcName) then
		return;
	end
	
	local isCompletable = IsQuestCompletable() == 1;
	local isDaily = QuestIsDaily() == 1 or QuestIsWeekly() == 1;
	local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
	
	if isCompletable and (isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily)) then
		CompleteQuest();
	end
end

function DailyGrind:QUEST_COMPLETE()
	local questTitle = self:SanitizeQuestTitle(GetTitleText());
	local npcName, realm = UnitName("npc");
	local isDaily = QuestIsDaily() == 1 or QuestIsWeekly() == 1;
	local isInHistory = self:IsInList(DailyGrindQuests, questTitle);
	
	--Attempt to turn in and record.
	if DailyGrindSettings.Enabled and not IsControlKeyDown() and not self:IsBlacklisted(questTitle) and not self:IsBlacklistedNpc(npcName) then
		if isInHistory or (self:GetAutoAcceptAllEnabled() and isDaily) then
			local numQuestChoices = GetNumQuestChoices();
			if numQuestChoices <= 1 then
				GetQuestReward(numQuestChoices);
			else
				self:SelectReward(questTitle);
			end
		end
	end
	
	if isDaily and not isInHistory then
		DailyGrindQuests[questTitle] = 1;	-- always record, even when disabled
	end
end

function DailyGrind:SelectReward(questTitle)
	local myChoices = self:GetMatchingQuestRewards();
	local numMyChoices = self:TableCount(myChoices);
	if numMyChoices == 1 then	-- Exactly one matching item. Happy path.
		local choiceIndex, itemName;
		for index, value in pairs(myChoices) do
			choiceIndex = index;
			itemName = value;
		end
		print(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => \""..itemName.."\" automatically selected.");
		GetQuestReward(choiceIndex);
	elseif numMyChoices > 1 then	-- 2+ matching items. Not so happy.
		print(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => Multiple matches found:\n");
		for index, value in pairs(myChoices) do	-- Build a string that displays all the conflicting rewards.
			print(" - "..value);
		end
		print("  Please choose your reward manually and then review your "..rewardListText..".\n");
	else	-- No matching items. Sad.
		print(addonAbbr.." "..rewardListText..": \""..questTitle.."\" => No matching rewards found; please choose manually.");
	end
end

function DailyGrind:ShowHelp()
	print("==]  "..self:GetStatus().."  [==\n");
	print("Dailies that you complete are added to a database. Whenever you see that daily again, you will auto-accept/complete it.");
	print("You may also set "..addonTitle.." to auto-accept ALL daily quests, whether or not you have seen them before, by using |c"..commandColor.."/dg all|r. Dailies that you complete with this option enabled will continue to be added to your database, so turning this option off later will not set you back.");
	print("Usage:  |c"..commandColor.."/dg <enable|on> <disable|off> <tog[gle]> <all> <reset>|r\n");
	print(rewardListText..": Set automatic reward preferences (|c"..commandColor.."/dg rl help|r for more info)");
	print(blacklistText..": Ignore specific quests (|c"..commandColor.."/dg bl help|r for more info)");
	print(npcBlacklistText..": Ignore quests from specific NPCs (|c"..commandColor.."/dg nbl help|r for more info)");
end
